/* action.c deals with the action buttons affect in various environments */

#include "rpg.h"

#define CHAT_BUBBLE_W 255
#define CHAT_BUBBLE_H 192

#define DIALOG_1 0 /* Quest not yet given */
#define DIALOG_2 1 /* Quest Complete */

#define MAX_CHAR 35

#define MAX_ANIMATION_STEPS 3 /* This will go up to 8 */

extern TTF_Font *merchant_font, *stats_font;
extern int global_x, global_y, ManPos;
extern SDL_Surface *man[8], *screen;
extern struct inventory inventory;
extern int ITEM_COUNT;
extern struct NPC npc[255];
extern struct quest_list *root;
extern struct hero hero;

struct dialog_vector
{
	char buf[20];
} dv[32];

struct text_surface ts2[32];

int format_dialog(char *buf, char dialog[][MAX_CHAR])
{
	SDL_Color font_fg = { 0xff, 0xff, 0xff, 0x00 };
	const int max_chars = 20;
	char *p;
	TTF_Font *dialog_font = stats_font;

	int lines = 0;
	int i, inc = MAX_CHAR;
	uint8_t last_cycle = 0;

	printf("buf: %s\n", buf);
	for (i = 0; i < 32; i++)
		memset(dv[i].buf, 0, max_chars);
	
	if ((p = strchr(buf, ';')))			
		*p = 0;
	if ((p = strchr(buf, '"')))
		*p = 0;

	p = buf;
	/* last two bytes are "; so we can do strlen(buf)-1 */
	while (1)
	{
		
		strncpy(dv[lines].buf, p, MAX_CHAR);
		dv[lines].buf[MAX_CHAR] = '\0';
		ts2[lines].s = TTF_RenderText_Blended (dialog_font, dv[lines].buf, font_fg);
		i += strlen(dv[lines].buf);
		p = buf + inc;
		inc += MAX_CHAR;
		lines++;	
		if (last_cycle == 1)	
			break;
		if (i > strlen(buf))
			last_cycle = 1;
	}
	return lines;
}

void talk2NPC(int ActiveWorld, int ActiveNPC, struct dialog_info *dialogs)
{
	SDL_Surface *dialog_box;	
	char dialog[32][MAX_CHAR];
	TTF_Font *dialog_font = stats_font;
	int alpha = SDL_ALPHA_TRANSPARENT;
	int x = global_x, y = global_y;
	int i, dc, qc, j, lines = 0;
        char *buf[8], quest_complete = 0;
	struct quest_list *current_quest;
	SDL_Event event;
	SDLKey key;
	uint8_t alldone = 0;

        SDL_Color font_fg = { 0xff, 0xff, 0xff, 0x00 };
        SDL_Color font_bg = { 0x00, 0x00, 0x00, 0x00 };

	dialog_box = LoadAlphaImage("bmp/misc/chat_bubble.bmp");
	SDL_SetAlpha (dialog_box, SDL_SRCALPHA, alpha + 100);
	
	for (i = 0; i < 8; i++)
		buf[i] = (char *)malloc(255);
#ifdef DEBUG
	printf("Loaded chat bubble - ActiveNPC: %d index %d - going through loop\n", ActiveNPC, dialogs[i].index);
#endif	

	for (i = 0; i < ActiveNPC; i++)
	{ 
		if (SDL_CollideBox(hero.walk[ManPos].image, x, y, npc[i].image[npc[i].direction][0], 
		    npc[i].x, npc[i].y, MAX_ANIMATION_STEPS, BBOX))
		{
			drawSprite(dialog_box, screen, 0, 0, npc[i].x - 100, npc[i].y - CHAT_BUBBLE_H, CHAT_BUBBLE_W, CHAT_BUBBLE_H);
			for (dc = 0; dc < dialogs[i].index; dc++)
				sprintf(buf[dc], "%s", dialogs[i].speech[dc]);
		
			for (current_quest = root; current_quest != NULL; current_quest = current_quest->next)
			{
				for (qc = 0; qc < npc[i].quest_count; qc++)
					if (current_quest->id == npc[i].quests[qc])
						if (current_quest->status == 1)
						{
							quest_complete = 1;
							//remove_quest(current_quest->id);
							break;
						}
				if (quest_complete)
					break;
			}
			if (!quest_complete)
				lines = format_dialog(buf[DIALOG_1], dialog);
			else
				lines = format_dialog(buf[DIALOG_2], dialog);
			if (lines == -1)
				do_panic();

			for (j = 0; j < lines; j++)
				drawSprite(ts2[j].s, screen, 0, 0, (npc[i].x - 90), (npc[i].y - CHAT_BUBBLE_H + 15) + j * 15, CHAT_BUBBLE_W - 80, 10);
			
			SDL_Flip(screen);
			SDL_Delay(250);
			
			/* EVENT LOOP */
			npc[i].InAction = 1;
			wait_loop:
			
			/* The SDL_Flip happens in draw_NPC every time npc->amc % npc->amv == 0 */
			ClearBlack(screen);
			DrawWorld(ActiveWorld);
			drawSprite(dialog_box, screen, 0, 0, npc[i].x - 100, npc[i].y - CHAT_BUBBLE_H, CHAT_BUBBLE_W, CHAT_BUBBLE_H);
			for (j = 0; j < lines; j++)
                        	drawSprite(ts2[j].s, screen, 0, 0, (npc[i].x - 90), (npc[i].y - CHAT_BUBBLE_H + 15) + j * 15, CHAT_BUBBLE_W - 80, 10);
		 	draw_NPC(ActiveNPC, x, y, ActiveWorld);

			SDL_PollEvent(&event);
                        switch(event.type)
                        {
                        	case SDL_KEYDOWN:
                                	key = event.key.keysym.sym;
                                        switch(key)
				 	{
						case SDLK_a:
							SDL_Delay(250);
							alldone = 1;
							break;		
						case SDLK_SPACE:
							SDL_Delay(250);
							alldone = 1;
							break;
						case SDLK_RETURN:
							SDL_Delay(250);
                                                        alldone = 1;
                                                        break;
					}
			}
			if (alldone)
				goto done;
			else
				goto wait_loop;
		
		}

		
	}
	done:
	ClearBlack(screen);
        DrawWorld(ActiveWorld); 
	npc[i].InAction = 0;
        draw_NPC(ActiveNPC, x, y, ActiveWorld);
	SDL_Flip(screen);

	for (i = 0; i < 8; i++)
		free(buf[i]);
}
void
talk2merchant (int ActiveHouse, struct player_stats *stats)
{
	struct text_surface ts[8];
	char *buf[8];
	int i, hp, mp, price;
	SDL_Surface *welcome, *confirm, *quit, *additem, *gold;
	int x = global_x, y = global_y;

	SDL_Color font_red = { 0xff, 0x00, 0x00, 0x00 };
	SDL_Color font_fg = { 0xff, 0xff, 0xff, 0x00 };
	SDL_Color font_bg = { 0x00, 0x00, 0x00, 0x00 };

	SDL_Event event, subevent;

	if (stats->level < 5)
	{
		mp = hp = 10;
		price = 15;
	}
	else if (stats->level >= 5 && stats->level <= 10)
	{
		mp = hp = 30;
		price = 50;
	}
	else if (stats->level >= 10 && stats->level <= 20)
	{
		mp = hp = 75;
		price = 100;
	}

	for (i = 0; i < 8; i++)
		if ((buf[i] = (char *) malloc (255)) == NULL)
		{
			perror ("malloc");
			exit (-1);
		}

	for (i = 0; i < 10; i++)
	{
		printf("ID: %d\n", inventory.items[i].item_id);
	}

	printf("HERE WE ARE: ActiveHouse: %d\n", ActiveHouse);
	switch (ActiveHouse)
	{
	case HOUSE_FLOOR1:
		sprintf (buf[0], "Welcome Lavren! Here is my inventory...");
		sprintf (buf[1], "1. Potion of HP -> HP: %d Price: %d", hp, price);
		sprintf (buf[2], "2. Potion of MP -> MP: %d Price: %d", mp, price);
		sprintf (buf[3], "Are you sure? (Y/N)");
		sprintf (buf[4], "Added item to inventory");
		sprintf (buf[5], "(Q)uit");
		sprintf (buf[6], "WHAT? Not enough gold? Get your fruit topping ass out of here!");

		welcome = TTF_RenderText_Blended (merchant_font, buf[0], font_fg);
		drawSprite (welcome, screen, 0, 0, 180, 10, 250, 10);
		SDL_Flip (screen);

		/* Create text surface msg's */
		confirm = TTF_RenderText_Blended (merchant_font, buf[3], font_fg);
		additem = TTF_RenderText_Blended (merchant_font, buf[4], font_fg);
		quit = TTF_RenderText_Blended (merchant_font, buf[5], font_fg);
		gold = TTF_RenderText_Blended (merchant_font, buf[6], font_fg);

		/* Delay before listing items */
		SDL_Delay (1500);

	list_items:
		for (i = 0; i <= 2; i++)
		{
			ts[i].s = TTF_RenderText_Blended (merchant_font, buf[i], font_fg);
			drawSprite (ts[i].s, screen, 0, 0, 180, 10 + i * 18, 250, 10);
		}
		drawSprite (quit, screen, 0, 0, 180, i * 21, 100, 10);
		SDL_Flip (screen);

		for (;;)
			if (SDL_PollEvent (&event))
			{
				if (testKey (SDLK_1))
				{
					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					drawSprite (confirm, screen, 0, 0, 180, 10, 100, 10);
					SDL_Flip (screen);

				wait_item_1:
					if (SDL_WaitEvent (&subevent))
					{
						if (testKey (SDLK_y))
							goto add_item_1;
	
						else if (testKey (SDLK_n))
						{
							ClearBlack (screen);
							DrawHouse ();
							DrawHero(&hero);
							goto list_items;
						}
						else
							goto wait_item_1;

					}
				add_item_1:
					/* ADDING ITEM TO INVENTORY */

					/* Do we have enough gold? */
					if ((signed)(stats->gold - price) < 0)
					{
						ClearBlack (screen);
                                        	DrawHouse ();
						DrawHero(&hero);
                                        	drawSprite (gold, screen, 0, 0, 180, 10, 100, 10);
						SDL_Flip(screen);
						SDL_Delay (1750);
						
						ClearBlack (screen);
                                        	DrawHouse ();
						DrawHero(&hero);
                                        	goto list_items;
					}
					
					ITEM_COUNT++;
					/* Subtract gold from player */
					stats->gold -= price;

					/* Add item */
					for (i = 0; i < MAX_ITEM; i++)
					{
						if (inventory.items[i].item_id == POTION_OF_LIFE_ID)
						{
							inventory.items[i].quanity++;
							break;
						}
						else if (inventory.items[i].item_id == 0)
						{
							inventory.items[i].item_id = POTION_OF_LIFE_ID;
							inventory.items[i].quanity++;
							break;
						}

					}
					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					drawSprite (additem, screen, 0, 0, 180, 10, 100, 10);
					SDL_Flip (screen);
					SDL_Delay (1000);

					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					goto list_items;
					break;

				}
				else if (testKey (SDLK_2))
				{
					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					drawSprite (confirm, screen, 0, 0, 180, 10, 100, 10);
					SDL_Flip (screen);

				wait_item_2:
					if (SDL_WaitEvent (&subevent))
					{
						if (testKey (SDLK_y))
							goto add_item_2;
						else if (testKey (SDLK_n))
						{
							ClearBlack (screen);
							DrawHouse ();
						        DrawHero(&hero);
							goto list_items;
						}
						else
							goto wait_item_2;

					}
				add_item_2:
					/* ADDING ITEM TO INVENTORY */

			                if ((signed)(stats->gold - price) < 0)
                                        {
                                                ClearBlack (screen);
                                                DrawHouse ();
						DrawHero(&hero);
                                                drawSprite (gold, screen, 0, 0, 180, 10, 100, 10);
                                                SDL_Flip(screen);
                                                SDL_Delay (1750);

                                                ClearBlack (screen);
                                                DrawHouse ();
						DrawHero(&hero);
                                                goto list_items;
                                        }
					
					ITEM_COUNT++;
					stats->gold -= price;
					for (i = 0; i < MAX_ITEM; i++)
					{
						if (inventory.items[i].item_id == POTION_OF_MAGIC_ID)
						{
							inventory.items[i].quanity++;
							break;
						}
						else if (inventory.items[i].item_id == 0)
						{
							inventory.items[i].item_id = POTION_OF_MAGIC_ID;
							inventory.items[i].quanity++;
							break;
						}
					}
					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					drawSprite (additem, screen, 0, 0, 180, 10, 100, 10);
					SDL_Flip (screen);
					SDL_Delay (1000);

					ClearBlack (screen);
					DrawHouse ();
					DrawHero(&hero);
					goto list_items;

				}
				else if (testKey (SDLK_q))
					goto finished;

			}

	}

finished:
	for (i = 0; i <= 2; i++)
		SDL_FreeSurface (ts[i].s);

	SDL_FreeSurface (additem);
	SDL_FreeSurface (quit);
	SDL_FreeSurface (welcome);
	SDL_FreeSurface (confirm);

	for (i = 0; i < 8; i++)
		free (buf[i]);

}
