/* This is a part of the custom game script interpreter designed for easily adding */
/* enemy, npc, and item content to Lavrens Oracle */

#include "rpg.h"

#define UNSET 0xffff
#define ENEMY_CONTENT_SPECIFIERS 10
#define NPC_CONTENT_SPECIFIERS 6

int cmpstr(char *buf, char *str, ...)
{
        char string[255];
        va_list va;

        va_start (va, str);
        vsnprintf (string, 255, str, va);
        va_end (va);

	if (buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1] = '\0';

	return strcasecmp(string, buf);
}

extern int ActiveEnemies;
extern int ActiveNPC;

/* This will load enemies, npc's, and quests */
int parse_dynamic_data(struct MonstXY *monster, struct NPC *npc, struct inventory *inventory, struct dialog_info *dialogs, int map_location)
{
	FILE *fd;
	char buf[255], *p, *q, *sp, *s;
	int i, j, enemy_count = 0, c = 0;
	uint8_t in_stanza = 0, end_stanza = 0;
	DIR *dd;
	struct dirent *d;
	struct enemy_values
	{
		int enemy_angles, hit_images, attack_images, walk_images, x, y;
		int hp, damage;
		char start_angle[32], image_directory[255], image_key[255], image_path[255], anim_speed[255], enemy_speed[255];
	} enemy;
	
	struct npc_values
	{
		int talk_images, x, y, image_angles, damage_increase, damage_intake;
		char name[255], image_directory[256], start_angle[12], item_name[255], image_path[256];
		
	} npc_tmp;
 
	uint8_t set_dialog = 0;
        uint8_t set_quest = 0;
        uint8_t added_quest = 0;
	uint8_t quest_count = 0;

	char imgname[255];
	/* north south east west */	
	int north = 0, south = 0, east = 0, west = 0;
	/* length of image key string */
	int key_len;
	
	int npc_count = 0;
	int quest_id = 0;

	/* 8 directions, 32 images per direction */
	char image_array[8][32][255];

	memset(enemy.anim_speed, 0, 255);
	memset(enemy.enemy_speed, 0, 255);

	enemy.enemy_angles = UNSET;
	enemy.hit_images = UNSET;
	enemy.attack_images = UNSET;
	enemy.walk_images = UNSET;
	enemy.x = UNSET;
	enemy.y = UNSET;
	enemy.hp = UNSET;
	enemy.damage = UNSET;

	npc_tmp.damage_intake = 0;
	npc_tmp.damage_increase = 0;

	if ((fd = fopen(MASTER_DATA, "r")) == NULL)
	{
		fprintf(stderr, "Unable to open %s - %s\n", MASTER_DATA, strerror(errno));
		return -1;
	}

	while (fgets(buf, sizeof(buf), fd))
	{
		if (buf[0] == '#' || buf[0] == '\n')
			continue;

		if ((strncasecmp(buf, "<location", 9) == 0) && in_stanza)
		{
			printf("GS error: %s - attempting to define a stanza without closing the previous one\n", MASTER_DATA);
			return -1;
		}
		
		/* The only thing that should be specified when not in a stanza */
		/* is declaring a new stanza, no actual game content */
		if (!in_stanza)	
		{
			if (cmpstr(buf, "<location %d>", map_location) == 0)
				in_stanza = 1;
			else
				continue;
		}
		else
		if (in_stanza)
		{
			if (strncasecmp(buf, "</location", 9) == 0)
			{
				for (p = buf; *p != '>'; p++);
				*(p + 1) = '\0';

				if (cmpstr(buf, "</location %d>", map_location) == 0)
				{
					in_stanza = 0;
					return 0;
				}
				else
				{
					printf("GS error: %s - attempting to end mismatched stanza\n", MASTER_DATA);
					return -1;
				}
			}
		}
		/* From this point on we should be in a stanza */
	
		s = buf;
		while (*s == 0x20)
			s++;

		if (*s == '\n')
			continue;
		
		if (strncasecmp(s, "enemy:", 6) == 0)
		{
			if (monster == NULL)
				continue;

			while (*s != ':')
				s++;
		 	s += 1;
			while (*s == 0x20)
				s++;


			for (i = 0; i < ENEMY_CONTENT_SPECIFIERS; i++)
			{
				if (i > 0)
				{
					while (*p == 0x20)
						p++;
					s = p;
					if ((p = strchr(s, '=')) == NULL)
                                        {
                                                printf("GS error: %s - unable to find content specifiers in enemy definition\n", MASTER_DATA);
                                                return -1;
                                        }
					*p = 0x20; 
					q = p + 1;
				}
				else
				{
					if ((p = strchr(s, '=')) == NULL)
					{
						printf("GS error: %s - unable to find content specifiers in enemy definition\n", MASTER_DATA);
						return -1;
					}
					*p = 0x20;
				 	q = p + 1;
				}

				p = strchr(q, ',');
				*p = '\0';
				
				/* enemy image directory (and enemy_name) */
				if (strncasecmp(s, "image_dir", 9) == 0)
					sscanf(s, "image_dir %s", enemy.image_directory);
				else
				if (strncasecmp(s, "image_angles", 12) == 0)
					sscanf(s, "image_angles %d", &enemy.enemy_angles);	
				else
				if (strncasecmp(s, "hit_images", 10) == 0)
					sscanf(s, "hit_images %d", &enemy.hit_images);
				else
				if (strncasecmp(s, "attack_images", 13) == 0)
					sscanf(s, "attack_images %d", &enemy.attack_images);
				else
				if (strncasecmp(s, "walk_images", 11) == 0)
					sscanf(s, "walk_images %d", &enemy.walk_images);
				else
				if (strncasecmp(s, "anim_speed", 10) == 0)
					sscanf(s, "anim_speed %s", &enemy.anim_speed);
				else
				if (strncasecmp(s, "enemy_speed", 11) == 0)
					sscanf(s, "enemy_speed %s", &enemy.enemy_speed);
				else
				if (strncasecmp(s, "pos", 3) == 0)
					sscanf(s, "pos %d-%d", &enemy.x, &enemy.y);
				else
				if (strncasecmp(s, "hp", 2) == 0)
					sscanf(s, "hp %d", &enemy.hp);
				else
				if (strncasecmp(s, "damage", 6) == 0)
					sscanf(s, "damage %d", &enemy.damage);
				else
				if (strncasecmp(s, "start_angle", 11) == 0)
					sscanf(s, "start_angle %s", &enemy.start_angle);
				
				else
				{
					if ((p = strchr(s, 0x20)))
						*p = '\0';
					printf("GS error: %s - unrecognized content specifier (%s)\n", MASTER_DATA, s);
					return -1;
				}

				p++;
								
			}
			verify_data:
			if (enemy.image_directory[0] == 0x20 || enemy.image_directory[0] == '\0')
			{
				printf("GS error: %s - missing image_directory in enemy definition\n", MASTER_DATA);
				return -1;
			}
				
			if (enemy.enemy_angles == UNSET)
			{
				printf("GS error: %s - missing image_angles in enemy definition\n", MASTER_DATA);
				return -1;
			}

			if (enemy.enemy_speed[0] == 0x20)
			{
				printf("GS error: %s - missing enemy_speed in enemy definition\n", MASTER_DATA);
				return -1;
			}

			if (enemy.hit_images == UNSET)
			{
				printf("GS error: %s - missing hit_images in enemy definition\n", MASTER_DATA);
				return -1;
			}
			
			if (enemy.walk_images == UNSET)
                        {
                                printf("GS error: %s - missing walk_images in enemy definition\n", MASTER_DATA);
                                return -1;
                        }
	
			if (enemy.attack_images == UNSET)
                        {
                                printf("GS error: %s - missing hit_images in enemy definition\n", MASTER_DATA);
                                return -1;
                        }
			
			if (enemy.x == UNSET || enemy.y == UNSET)
			{
				printf("GS error: %s - missing pos in enemy definition (both x and y must be set)\n", MASTER_DATA);
				return -1;
			}
			
			if (enemy.hp == UNSET)
			{
				printf("GS error: %s - missing hp in enemy definition\n", MASTER_DATA);
				return -1;
			}
			
			if (enemy.damage == UNSET)
			{
				printf("GS error: %s - missing damage in enemy definition\n", MASTER_DATA);
				return -1;
			}

			if (enemy.start_angle[0] == '\0' || enemy.start_angle[0] == 0x20)
			{
				printf("GS error: %s - missing start_angle in enemy definition\n", MASTER_DATA);
				return -1;
			}
			
			/* If we've gotten here then we can successfully feel a monster struct */
			monster[enemy_count].x = enemy.x;
			monster[enemy_count].y = enemy.y;
			monster[enemy_count].image_count = enemy.walk_images;
			monster[enemy_count].hit_image_count = enemy.hit_images;
			monster[enemy_count].attack_image_count = enemy.attack_images;
			monster[enemy_count].damage = enemy.damage;
			monster[enemy_count].max_hp = monster[enemy_count].hp = enemy.hp;
			monster[enemy_count].multi_directional = 1;
			monster[enemy_count].status = 1;

			if (strcasecmp(enemy.start_angle, "south") == 0)
				monster[enemy_count].direction = S;
			else
			if (strcasecmp(enemy.start_angle, "north") == 0)
                                monster[enemy_count].direction = N;
			else
			if (strcasecmp(enemy.start_angle, "east") == 0)
                                monster[enemy_count].direction = E;
			else
			if (strcasecmp(enemy.start_angle, "east") == 0)
                                monster[enemy_count].direction = W;
			
			/* Set animation speed */
			if (strcasecmp(enemy.anim_speed, "fast") == 0)
			{
				monster[enemy_count].amv = FAST_SPEED_MODULO;
				monster[enemy_count].hit_amv = FAST_SPEED_MODULO;
				monster[enemy_count].attack_amv = FAST_SPEED_MODULO;
			}
			else
			if (strcasecmp(enemy.anim_speed, "medium") == 0)
			{
				
				monster[enemy_count].amv = MEDIUM_SPEED_MODULO;
                                monster[enemy_count].hit_amv = MEDIUM_SPEED_MODULO;
                                monster[enemy_count].attack_amv = MEDIUM_SPEED_MODULO;
			}
			else
			if (strcasecmp(enemy.anim_speed, "slow") == 0)
                        {
				monster[enemy_count].amv = SLOW_SPEED_MODULO;
                                monster[enemy_count].hit_amv = SLOW_SPEED_MODULO;
                                monster[enemy_count].attack_amv = SLOW_SPEED_MODULO;
			}
			
		        if ((dd = opendir(enemy.image_directory)) == NULL)
                		if (errno == ENOTDIR)
                		{
					printf("GS error: %s - specified enemy image directory is incorrect (directory does not exist)\n", MASTER_DATA);
					return -1;
                                }
               		 else
                        	return -1;
	
			int w, h;
			/* We will use the directory name as its image key */
			if ((p = strchr(enemy.image_directory, '/')))
				p++;
			strcpy(enemy.image_key, p);
			key_len = strlen(enemy.image_key);

			north = 0, south = 0, east = 0, west = 0;

			/* Load walk images */
			if (enemy.walk_images)
			while((d = readdir(dd)) != NULL)
                	{
				if (d->d_name[0] == '.')
					continue;
                        	strncpy(enemy.image_path, enemy.image_directory, sizeof(enemy.image_path)/2);
				enemy.image_path[strlen(enemy.image_path)] = '/';
				strncat(enemy.image_path, d->d_name, sizeof(enemy.image_path)/2);
				enemy.image_path[sizeof(enemy.image_path) - 1] = '\0';
				
				strncpy(imgname, d->d_name, sizeof(imgname));
				imgname[sizeof(imgname) - 1] = '\0';
				
				/* We aren't interested in the hit or attack images */
				if (strstr(imgname, "hit") || strstr(imgname, "attack"))
					continue;
				
				p = imgname;
				for (i = 0; i < key_len; i++)
					p++;
				
				/* *p should == '_' if the correct convention is being followed */
				if (*p++ != '_')
				{
					printf("Image name: %s should be of the format <image_X#> where X is N, S, E, or W\n", imgname);
					return -1;
				}
				
				/* There might be 8 images, but maybe we're only using 4 */
				if (north > monster[enemy_count].image_count)
					continue;
				if (south > monster[enemy_count].image_count)
					continue;
				if (east > monster[enemy_count].image_count)
					continue;
				if (west > monster[enemy_count].image_count)
					continue;

				if (*p == 'n')
				{
					if (*(p + 1) == '1')
						strcpy(image_array[N][0], enemy.image_path);
					else
					if (*(p + 1) == '2')
						strcpy(image_array[N][1], enemy.image_path);
					else
					if (*(p + 1) == '3')
                                                strcpy(image_array[N][2], enemy.image_path);
					else
					if (*(p + 1) == '4')
                                                strcpy(image_array[N][3], enemy.image_path);
                                        else
                                        if (*(p + 1) == '5')
                                                strcpy(image_array[N][4], enemy.image_path);
                                        else
                                        if (*(p + 1) == '6')
                                                strcpy(image_array[N][5], enemy.image_path);
					else
					if (*(p + 1) == '7')
                                                strcpy(image_array[N][6], enemy.image_path);
                                        else
                                        if (*(p + 1) == '8')
                                                strcpy(image_array[N][7], enemy.image_path);
					
					if (north == 0)
					{
						get_bmp_size(enemy.image_path, &monster[enemy_count].size[N].w, &monster[enemy_count].size[N].h);
						monster[enemy_count].image = LoadImage(enemy.image_path);
					}
				}
				else
				if (*p == 's')
				{	
                                        if (*(p + 1) == '1')
                                                strcpy(image_array[S][0], enemy.image_path);
                                        else
                                        if (*(p + 1) == '2')
                                                strcpy(image_array[S][1], enemy.image_path);
                                        else
                                        if (*(p + 1) == '3')
                                                strcpy(image_array[S][2], enemy.image_path);
                                        else
                                        if (*(p + 1) == '4')
                                                strcpy(image_array[S][3], enemy.image_path);
                                        else
                                        if (*(p + 1) == '5')
                                                strcpy(image_array[S][4], enemy.image_path);
                                        else
                                        if (*(p + 1) == '6')
                                                strcpy(image_array[S][5], enemy.image_path);
                                        else
                                        if (*(p + 1) == '7')
                                                strcpy(image_array[S][6], enemy.image_path);
                                        else
                                        if (*(p + 1) == '8')
                                                strcpy(image_array[S][7], enemy.image_path);
					if (south == 0)
						get_bmp_size(enemy.image_path,  &monster[enemy_count].size[S].w, &monster[enemy_count].size[S].h);
				}
				else
				if (*p == 'e')
				{
					 if (*(p + 1) == '1')
                                                strcpy(image_array[E][0], enemy.image_path);
                                        else
                                        if (*(p + 1) == '2')
                                                strcpy(image_array[E][1], enemy.image_path);
                                        else
                                        if (*(p + 1) == '3')
                                                strcpy(image_array[E][2], enemy.image_path);
                                        else
                                        if (*(p + 1) == '4')
                                                strcpy(image_array[E][3], enemy.image_path);
                                        else
                                        if (*(p + 1) == '5')
                                                strcpy(image_array[E][4], enemy.image_path);
                                        else
                                        if (*(p + 1) == '6')
                                                strcpy(image_array[E][5], enemy.image_path);
                                        else
                                        if (*(p + 1) == '7')
                                                strcpy(image_array[E][6], enemy.image_path);
                                        else
                                        if (*(p + 1) == '8')
                                                strcpy(image_array[E][7], enemy.image_path);

					if (east == 0)
						get_bmp_size(enemy.image_path, &monster[enemy_count].size[E].w, &monster[enemy_count].size[E].h);
				}
				else
				if (*p == 'w')
				{
				        if (*(p + 1) == '1')
                                                strcpy(image_array[W][0], enemy.image_path);
                                        else
                                        if (*(p + 1) == '2')
                                                strcpy(image_array[W][1], enemy.image_path);
                                        else
                                        if (*(p + 1) == '3')
                                                strcpy(image_array[W][2], enemy.image_path);
                                        else
                                        if (*(p + 1) == '4')
                                                strcpy(image_array[W][3], enemy.image_path);
                                        else
                                        if (*(p + 1) == '5')
                                                strcpy(image_array[W][4], enemy.image_path);
                                        else
                                        if (*(p + 1) == '6')
                                                strcpy(image_array[W][5], enemy.image_path);
                                        else
                                        if (*(p + 1) == '7')
                                                strcpy(image_array[W][6], enemy.image_path);
                                        else
                                        if (*(p + 1) == '8')
                                                strcpy(image_array[W][7], enemy.image_path);
					if (west == 0)
						get_bmp_size(enemy.image_path, &monster[enemy_count].size[W].w, &monster[enemy_count].size[W].h);
				}
				memset(enemy.image_path, 0, 255);
				memset(imgname, 0, 255);
			}
			/*
			for (i = 0; i < monster[enemy_count].image_count; i++)
                        {
				printf("%s\n%s\n%s\n%s\n",
                                image_array[N][i],image_array[S][i],image_array[E][i],image_array[W][i]);
                        }
			*/
			for (i = 0; i < monster[enemy_count].image_count; i++)
			{
				monster[enemy_count].animation[N][i] = LoadImage(image_array[N][i]);
				monster[enemy_count].animation[S][i] = LoadImage(image_array[S][i]);
 				monster[enemy_count].animation[E][i] = LoadImage(image_array[E][i]);
                                monster[enemy_count].animation[W][i] = LoadImage(image_array[W][i]);
			}

			/* Re-opening the same directory 3 times is stupid yes, we'll find another way */
			closedir(dd);
		        dd = opendir(enemy.image_directory);
			for (i = 0; i < monster[enemy_count].image_count; i++)
			{
				memset(image_array[N][i], 0, sizeof(image_array[N][i]));
				memset(image_array[N][i], 0, sizeof(image_array[N][i]));
				memset(image_array[N][i], 0, sizeof(image_array[N][i]));
				memset(image_array[N][i], 0, sizeof(image_array[N][i]));
			}
		        north = 0, south = 0, east = 0, west = 0;
			if (enemy.hit_images)
		        while((d = readdir(dd)) != NULL)
                        {
				if (d->d_name[0] == '.')
					continue;
                                strncpy(enemy.image_path, enemy.image_directory, sizeof(enemy.image_path)/2);
                                enemy.image_path[strlen(enemy.image_path)] = '/';
				strncat(enemy.image_path, d->d_name, sizeof(enemy.image_path)/2);
                                enemy.image_path[sizeof(enemy.image_path) - 1] = '\0';

                                strncpy(imgname, d->d_name, sizeof(imgname));
                                imgname[sizeof(imgname) - 1] = '\0';

                                /* We aren't interested in the hit or attack images */
                                if (strstr(imgname, "hit")) 
				{

                                	p = imgname;
                                	for (i = 0; i < key_len; i++)
                                        	p++;
					if (strncmp(p, "_hit_", 5) != 0)
					{
						printf("Image name: %s should be of the format <image_hit_X#> where X is N, S, E, or W\n", imgname);
                                                return -1;
					}
					
					p += 5;

                                	if (*p == 'n')
					{
	                                	if (*(p + 1) == '1')
                                                	strcpy(image_array[N][0], enemy.image_path);
                                       		else
                                        	if (*(p + 1) == '2')
                                                	strcpy(image_array[N][1], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '3')
                                                	strcpy(image_array[N][2], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '4')
                                               		strcpy(image_array[N][3], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '5')
                                                	strcpy(image_array[N][4], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '6')
                                                	strcpy(image_array[N][5], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '7')
                                                	strcpy(image_array[N][6], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '8')
                                                	strcpy(image_array[N][7], enemy.image_path);
						
					}
                               		else
					if (*p == 's')
                                	{
                                        	if (*(p + 1) == '1')
                                                	strcpy(image_array[S][0], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '2')
                                                	strcpy(image_array[S][1], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '3')
                                                	strcpy(image_array[S][2], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '4')
                                                	strcpy(image_array[S][3], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '5')
                                                	strcpy(image_array[S][4], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '6')
                                               		strcpy(image_array[S][5], enemy.image_path);
                                       		else
                                        	if (*(p + 1) == '7')
                                                	strcpy(image_array[S][6], enemy.image_path);
                                       	 	else
                                        	if (*(p + 1) == '8')
                                                	strcpy(image_array[S][7], enemy.image_path);

                                	}
					else
                               		if (*p == 'e')
                                	{
                                        	if (*(p + 1) == '1')
                                                	strcpy(image_array[E][0], enemy.image_path);
                                       	 	else
                                        	if (*(p + 1) == '2')
                                                	strcpy(image_array[E][1], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '3')
                                                	strcpy(image_array[E][2], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '4')
                                                	strcpy(image_array[E][3], enemy.image_path);
                                        	else
                                       	 	if (*(p + 1) == '5')
                                                	strcpy(image_array[E][4], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '6')
                                                	strcpy(image_array[E][5], enemy.image_path);
                                       	 	else
                                        	if (*(p + 1) == '7')
                                                	strcpy(image_array[E][6], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '8')
                                                	strcpy(image_array[E][7], enemy.image_path);


                                	}
                                	else
                                	if (*p == 'w')
                                	{
                                        	if (*(p + 1) == '1')
                                                	strcpy(image_array[W][0], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '2')
                                                	strcpy(image_array[W][1], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '3')
                                                	strcpy(image_array[W][2], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '4')
                                                	strcpy(image_array[W][3], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '5')
                                                	strcpy(image_array[W][4], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '6')
                                                	strcpy(image_array[W][5], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '7')
                                                	strcpy(image_array[W][6], enemy.image_path);
                                        	else
                                        	if (*(p + 1) == '8')
                                                	strcpy(image_array[W][7], enemy.image_path);
							

					}
					
                        	}
				memset(enemy.image_path, 0, 255);
                                memset(imgname, 0, 255);

			}
			/*
			for (i = 0; i < monster[enemy_count].image_count; i++)
                        {
                                printf("%s\n%s\n%s\n%s\n",
                                image_array[N][i],image_array[S][i],image_array[E][i],image_array[W][i]);
                        }
			*/

			for (i = 0; i < monster[enemy_count].image_count; i++)
                        {
                                monster[enemy_count].animation_hit[N][i] = LoadImage(image_array[N][i]);
                                monster[enemy_count].animation_hit[S][i] = LoadImage(image_array[S][i]);
                                monster[enemy_count].animation_hit[E][i] = LoadImage(image_array[E][i]);
                                monster[enemy_count].animation_hit[W][i] = LoadImage(image_array[W][i]);
                        }

                        for (i = 0; i < monster[enemy_count].image_count; i++)
                        {
                                memset(image_array[N][i], 0, sizeof(image_array[N][i]));
                                memset(image_array[N][i], 0, sizeof(image_array[N][i]));
                                memset(image_array[N][i], 0, sizeof(image_array[N][i]));
                                memset(image_array[N][i], 0, sizeof(image_array[N][i]));
                        }

			closedir(dd);
                        dd = opendir(enemy.image_directory);
			
			north = 0, south = 0, east = 0, west = 0;
                        
			if (enemy.attack_images)
                        while((d = readdir(dd)) != NULL)
                        {
				if (d->d_name[0] == '.')
					continue;

                                strncpy(enemy.image_path, enemy.image_directory, sizeof(enemy.image_path)/2);
                                enemy.image_path[strlen(enemy.image_path)] = '/';
				strncat(enemy.image_path, d->d_name, sizeof(enemy.image_path)/2);
                                enemy.image_path[sizeof(enemy.image_path) - 1] = '\0';

                                strncpy(imgname, d->d_name, sizeof(imgname));
                                imgname[sizeof(imgname) - 1] = '\0';

                                /* We aren't interested in the hit or attack images */
                                if (strstr(imgname, "attack"))
                                {

                                        p = imgname;
                                        for (i = 0; i < key_len; i++)
                                                p++;

                                        if (strncmp(p, "_attack_", 8) != 0)
                                        {
                                                printf("Image name: %s should be of the format <image_attack_X#> where X is N, S, E, or W\n", imgname);
                                                return -1;
                                        }

                                        p += 5;

                                        /* There might be 8 images, but maybe we're only using 4 */
                                        if (north > monster[enemy_count].image_count)
                                                continue;
                                        if (south > monster[enemy_count].image_count)
                                                continue;
                                        if (east > monster[enemy_count].image_count)
                                                continue;
                                        if (west > monster[enemy_count].image_count)
                                                continue;

                                
					if (*p == 'n')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[N][0], enemy.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[N][1], enemy.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[N][2], enemy.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[N][3], enemy.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[N][4], enemy.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[N][5], enemy.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[N][6], enemy.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[N][7], enemy.image_path);

                                        }
                                        else
                                        if (*p == 's')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[S][0], enemy.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[S][1], enemy.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[S][2], enemy.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[S][3], enemy.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[S][4], enemy.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[S][5], enemy.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[S][6], enemy.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[S][7], enemy.image_path);
                                        }
					else
					if (*p == 'e')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[E][0], enemy.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[E][1], enemy.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[E][2], enemy.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[E][3], enemy.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[E][4], enemy.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[E][5], enemy.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[E][6], enemy.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[E][7], enemy.image_path);

                                        }
                                        else
                                        if (*p == 'w')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[W][0], enemy.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[W][1], enemy.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[W][2], enemy.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[W][3], enemy.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[W][4], enemy.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[W][5], enemy.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[W][6], enemy.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[W][7], enemy.image_path);
                                        }


				}
                                memset(enemy.image_path, 0, 255);
                                memset(imgname, 0, 255);

                        }
			if (enemy.attack_images)
			{
				for (i = 0; i < monster[enemy_count].image_count; i++)
                        	{
                                	monster[enemy_count].animation_attack[N][i] = LoadImage(image_array[N][i]);
                               	 	monster[enemy_count].animation_attack[S][i] = LoadImage(image_array[S][i]);
                                	monster[enemy_count].animation_attack[E][i] = LoadImage(image_array[E][i]);
                                	monster[enemy_count].animation_attack[W][i] = LoadImage(image_array[W][i]);
				}
				closedir(dd);
			}
			enemy_count++;
			ActiveEnemies = enemy_count;
	        }	
		else
		/* NON PLAYABLE CHARACTER CREATION */
		if (strncasecmp(s, "npc_start", 9) == 0)
		{
			printf("Parsing NPC data\n");
			
			if (npc == NULL)
				continue;

			set_dialog = 0;
			set_quest = 0;
			added_quest = 0;
			
			/* Beginning NPC creation */
			fgets(buf, sizeof(buf), fd);
			s = buf;

			for (i = 0; i < NPC_CONTENT_SPECIFIERS; i++)
                        {
			 	if (i > 0)
                                {
                                        while (*p == 0x20)
                                                p++;
                                        s = p;
                                        if ((p = strchr(s, '=')) == NULL)
                                        {
                                                printf("GS error: %s - unable to find content specifiers in NPC definition\n", MASTER_DATA);
                                                return -1;
                                        }
                                        *p = 0x20;
                                        q = p + 1;
                                }
                                else
                                {
                                        if ((p = strchr(s, '=')) == NULL)
                                        {
                                                printf("GS error: %s - unable to find content specifiers in NPC definition\n", MASTER_DATA);
                                                return -1;
                                        }
                                        *p = 0x20;
                                        q = p + 1;
                                }

                                p = strchr(q, ',');
                                *p = '\0';
	
				if (strncasecmp(s, "name", 4) == 0)
                                        sscanf(s, "name %s", npc_tmp.name);
                                else
                                if (strncasecmp(s, "coordinates", 11) == 0)
                                        sscanf(s, "coordinates %d-%d", &npc_tmp.x, &npc_tmp.y);
				else
				if (strncasecmp(s, "image_dir", 9) == 0)
                                        sscanf(s, "image_dir %s", npc_tmp.image_directory);
                                else
                                if (strncasecmp(s, "image_angles", 12) == 0)
                                        sscanf(s, "image_angles %d", &npc_tmp.image_angles);
				else
				if (strncasecmp(s, "talk_images", 11) == 0)
                                        sscanf(s, "talk_images %d", &npc_tmp.talk_images);
				else
				if (strncasecmp(s, "start_angle", 11) == 0)
					sscanf(s, "start_angle %s", npc_tmp.start_angle);
				
				else
                                {
                                        if ((p = strchr(s, 0x20)))
                                                *p = '\0';
                                        printf("GS error: %s - unrecognized content specifier (%s)\n", MASTER_DATA, s);
                                        return -1;
                                }

                                p++;
			}
			
			
			npc[npc_count].x = npc_tmp.x;
			npc[npc_count].y = npc_tmp.y;
			
			if (strcasecmp(npc_tmp.start_angle, "south") == 0)
                                npc[npc_count].direction = S;
                        else
                        if (strcasecmp(npc_tmp.start_angle, "north") == 0)
                                npc[npc_count].direction = N;
                        else
                        if (strcasecmp(npc_tmp.start_angle, "east") == 0)
                                npc[npc_count].direction = E;
                        else
                        if (strcasecmp(npc_tmp.start_angle, "east") == 0)
                                npc[npc_count].direction = W;
			
				
			if ((dd = opendir(npc_tmp.image_directory)) == NULL)
                                if (errno == ENOTDIR || errno == ENOENT)
                                {
                                        printf("GS error: %s - specified NPC image directory is incorrect (directory does not exist)\n", MASTER_DATA);
                                        return -1;
                                }
                         else
                                return -1;

			/*
				NPC's need to be in bmp/npc 
			*/
			if ((p = strchr(npc_tmp.image_directory, '/')))
                                p++;
			while (*p != '/')
				p++;
                        p++;
			/* image key is just the name that all of the NPC's (or enemies) images start with */

			strcpy(npc[npc_count].image_key, p);
                        key_len = strlen(npc[npc_count].image_key);
			north = south = east = west = 0;

                        /* Load walk images */
                        while((d = readdir(dd)) != NULL)
                        {
                                if (d->d_name[0] == '.')
                                        continue;
                                strncpy(npc_tmp.image_path, npc_tmp.image_directory, sizeof(npc_tmp.image_path)/2);
                                npc_tmp.image_path[strlen(npc_tmp.image_path)] = '/';
                                strncat(npc_tmp.image_path, d->d_name, sizeof(npc_tmp.image_path)/2);
                                npc_tmp.image_path[sizeof(npc_tmp.image_path) - 1] = '\0';
                                
                                strncpy(imgname, d->d_name, sizeof(imgname));
                                imgname[sizeof(imgname) - 1] = '\0';
                                
                                /* We aren't interested in the hit or attack images */
                                if (!strstr(imgname, "talking"))
                                        continue;
				
				p = imgname;
                                for (i = 0; i < key_len; i++)
                                        p++;
                                
                                /* *p should == '_' if the correct convention is being followed */
                                
				if (strncasecmp(p, "_talking_", 9) != 0)
                                {
                                        printf("Image name: %s should be of the format <image_talking_X#> where X is N, S, E, or W\n", imgname);
                                        return -1;
                                }
				
					p++;
					while (*p != '_')
						p++;
					p += 1;

				 	if (*p == 'n')
                                 	{
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[N][0], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[N][1],  npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[N][2], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[N][3], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[N][4], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[N][5], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[N][6], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[N][7], npc_tmp.image_path);
						if (north == 0)
						{
							get_bmp_size(npc_tmp.image_path, &npc[npc_count].size[N].w, &npc[npc_count].size[N].h);
                                        		north++;
						}
						
					}
                                        else
                                        if (*p == 's')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[S][0], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[S][1], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[S][2], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[S][3], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[S][4], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[S][5], npc_tmp.image_path);
						 else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[S][6], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[S][7], npc_tmp.image_path);

						if (south == 0)
                                                {
                                                        get_bmp_size(npc_tmp.image_path, &npc[npc_count].size[S].w, &npc[npc_count].size[S].h);
                                                        south++;
                                                }

                                        }
                                        else
                                        if (*p == 'e')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[E][0], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[E][1], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[E][2], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '4')
                                                        strcpy(image_array[E][3], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[E][4], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[E][5], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[E][6], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[E][7], npc_tmp.image_path);

                                        	if (east == 0)
                                                {
                                                        get_bmp_size(npc_tmp.image_path, &npc[npc_count].size[E].w, &npc[npc_count].size[E].h);
                                                        east++;
                                                }


					}
                                        else
                                        if (*p == 'w')
                                        {
                                                if (*(p + 1) == '1')
                                                        strcpy(image_array[W][0], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '2')
                                                        strcpy(image_array[W][1], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '3')
                                                        strcpy(image_array[W][2], npc_tmp.image_path);
                                                else
						 if (*(p + 1) == '4')
                                                        strcpy(image_array[W][3], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '5')
                                                        strcpy(image_array[W][4], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '6')
                                                        strcpy(image_array[W][5], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '7')
                                                        strcpy(image_array[W][6], npc_tmp.image_path);
                                                else
                                                if (*(p + 1) == '8')
                                                        strcpy(image_array[W][7], npc_tmp.image_path);

						if (west == 0)
                                                {
                                                        get_bmp_size(npc_tmp.image_path, &npc[npc_count].size[W].w, &npc[npc_count].size[W].h);
                                                        west++;
                                                }

					}
                                        
                                memset(npc_tmp.image_path, 0, 255);
                                memset(imgname, 0, 255);
                        }

			for (i = 0; i < npc_tmp.talk_images; i++)
                        {
                                npc[npc_count].image[N][i] = LoadImage(image_array[N][i]);
                                npc[npc_count].image[S][i] = LoadImage(image_array[S][i]);
                                npc[npc_count].image[E][i] = LoadImage(image_array[E][i]);
                                npc[npc_count].image[W][i] = LoadImage(image_array[W][i]);
                        }
	
			npc[npc_count].talk_images = npc_tmp.talk_images;

                        closedir(dd);
			
			npc[npc_count].location = map_location;

			for(;;) 
			{
				fgets (buf, sizeof(buf), fd);
				
				if (strncasecmp(buf, "npc_end", 7) == 0)
				{
					if (set_dialog == 0)
					{
						printf("GS error: %s - NPC Creation requires atleast one dialog be set\n", MASTER_DATA);
						return -1;
					}
					/* NPC Creation complete */
					break;
				}
				else
				if (strncasecmp(buf, "dialog", 6) == 0)
				{
					set_dialog++;
					if (set_quest == 0 && set_dialog > 1)
					{
						printf("GS error: %s - More than one dialog should not exist unless it is quest driven\n", MASTER_DATA);
						return -1;
					}
					
					if ((p = strchr(buf, '"')) == NULL)
					{
						printf("GS error: %s - Missing opening '\"' around the dialog text\n");
						return -1;
					}
					
					s = p + 1;
					
					if ((p = strchr(buf, '"')) == NULL)
                                        {
                                                printf("GS error: %s - Missing closing '\"' around the dialog text in NPC dialog specifier\n");
                                                return -1;
                                        }

					*p = '\0';
					dialogs[npc_count].speech[dialogs[npc_count].index++] = strdup(s);
				}
				else
				if (strncasecmp(buf, "quest(", 6) == 0)
				{
					if ((p = strchr(buf, ')')) == NULL)
					{
						printf("GS error: %s - Missing closing bracket around the quest number in NPC quest specifier\n");
						return -1;
					}
					
					set_quest++;
					
					/* Get the next line which should be a quest descriptor */
					fgets (buf, sizeof(buf), fd);
					s = buf;

                        		for (i = 0; i < 3; i++)
                        		{
                                		if (i > 0)
                                		{
							p = sp;
                                        		while (*p == 0x20)
                                                		p++;
                                        		s = p;
                                        		if ((p = strchr(s, '=')) == NULL)
                                        		{
                                                		printf("GS error: %s - unable to find content specifiers in NPC definition\n", MASTER_DATA);
                                                		return -1;
                                        		}
                                        		*p = 0x20;
                                        		q = p + 1;
                                		}
                                		else
                                		{
                                       			if ((p = strchr(s, '=')) == NULL)
                                        		{
                                                		printf("GS error: %s - unable to find content specifiers in NPC definition\n", MASTER_DATA);
                                                		return -1;
                                        		}
                                        		*p = 0x20;
                                        		q = p + 1;
                                		}
                                		if ((p = strchr(q, ',')) == NULL)
						{
							printf("GS error: %s - the end of each content specifier requires a comma\n", MASTER_DATA);
							return -1;
						}
                                		*p = '\0';
						/* sp is save pointer */
						sp = p + 1;

                               	 		if (strncasecmp(s, "item", 4) == 0)
						{
							p = strchr(s, '[');
							if (!p)
							{
								printf("GS error: %s - Missing openening bracket to item description\n", MASTER_DATA);
								return -1;
							}
							
							*p = '\0';
                                        		sscanf(s, "item %s", npc_tmp.item_name);
							s = p + 1;
							p = strchr(s, '=');
							if (!p)
							{
								printf("GS error: %s - Missing item value assignment\n", MASTER_DATA);
								return -1;
							}
							*p = '\0';
							if (strncasecmp(s, "damage_increase", 15) == 0)
								sscanf(s, "damage_increase %d", &npc_tmp.damage_increase);	
							else
							if (strncasecmp(s, "damage_intake", 13) == 0)
                                                                sscanf(s, "damage_intake %d", &npc_tmp.damage_intake);
								
						}
						else
						if (strncasecmp(s, "required_quest", 14) == 0)
						{
							
							s += 14;

							p = strchr(s, '[');
							if (!p)
                                                        {
                                                                printf("GS error: %s - Missing openening bracket to quest name\n", MASTER_DATA);
                                                                return -1;      
                                                        }
							*p = '\0';

							
							quest_id = atoi(s);
							s++;

							while (*s == 0x20)
								s++;

							/* 's' should be the quest name */	
							s += 2;
							p = strchr(s, ']');
							if (!p)
							{
								printf("GS error: %s - Missing closing bracket to quest name\n", MASTER_DATA);
								return -1;
							}
							*p = '\0';
#ifdef DEBUG
							printf("Adding quest %s [%d]\n", s, quest_id);
	
#endif					
							npc[npc_count].quests[quest_count++] = quest_id;
							npc[npc_count].quest_count = quest_count;
							add_quest(s, quest_id);
						}
					}
		       		}	
			}
			npc_count++;
                        ActiveNPC = npc_count;

			
		}
    	 }
}
				
				
			
			
									
		

