
/* inventory.c - code for displaying inventory
 * Ryan O'Neill
 */

#include <stdio.h>
#include "rpg.h"

#define ACTION_ITEMS "data/action_items"

extern SDL_Surface *lavren;

void init_action_items(struct action_item_list *action_item)
{

	FILE *fd;
	char buf[MAX_LINE];
	char *p;
	int i;

	if ((fd = fopen(ACTION_ITEMS, "r")) == NULL)
	{
		printf("Error: Unable to open %s -> %s\n", ACTION_ITEMS, strerror(errno));
		exit(-1);
	}
	
	while (fgets(buf, sizeof(buf)-1, fd))
	{
		if (buf[0] == '#' || buf[0] == '\n')
			continue;
		if (strncasecmp(buf, "swords", 6) == 0)
		{
			p = strtok(buf, ":");
			for (i = 0; i <= atoi(p); i++)
			{
				fgets(buf, sizeof(buf)-1, fd);
				p = strtok(buf, ":");
				action_item->sword[i].id = atoi(p);
				p = strtok(NULL, ":");
				action_item->sword[i].damage = atoi(p);
				p = strtok(NULL, "\n");
				action_item->sword[i].name = strdup(p);
			}
		}
		else
		if (strncasecmp(buf, "armor", 5) == 0)
		{
			p = strtok(buf, ":");
                        for (i = 0; i <= atoi(p); i++)
                        {
                                fgets(buf, sizeof(buf)-1, fd);
                                p = strtok(buf, ":");
                                action_item->armor[i].id = atoi(p);
                                p = strtok(NULL, ":");
                                action_item->armor[i].damage_intake = atoi(p);
                                p = strtok(NULL, "\n");
                                action_item->armor[i].name = strdup(p);
                        }
                }
		else
		if (strncasecmp(buf, "spells", 5) == 0)
                {
                        p = strtok(buf, ":");
                        for (i = 0; i <= atoi(p); i++)
                        {
                                fgets(buf, sizeof(buf)-1, fd);
                                p = strtok(buf, ":");
                                action_item->spell[i].id = atoi(p);
                                p = strtok(NULL, ":");
                                action_item->spell[i].damage = atoi(p);
                                p = strtok(NULL, ":");
                                action_item->spell[i].rmp = atoi(p);
				p = strtok(NULL, "\n");
				action_item->spell[i].name = strdup(p);

                        }
                }
	  }	
	
	/* We start out with this spell */
	action_item->spell[0].attained = 1;
	
		
}

/* This function interacts with an event driven loop in main.c to handle inventory viewing and actions */
int
ListInventory (TTF_Font * item_font, TTF_Font * inventory_font,
	       struct inventory *inventory, char *player, SDL_Surface * screen, int8_t cursor, int8_t action, struct player_stats *stats)
{

	extern int INVENTORY_UPDATE, ITEM_COUNT;
	extern TTF_Font *stats_font;
	extern struct action_item_list action_item_list;

	struct item_and_cursor_bond
	{
		int item_id;
		int position[2];
	} item_cursor[MAX_ITEM];

	struct spell_and_cursor_bond
	{
		int spell_id;
		int position[2];
	} spell_item_cursor[MAX_ITEM];

	int i, j, count = 0, spell_count = 0;
	struct text_surface ts[MAX_ITEM], ts2[MAX_SPELLS];
	static uint8_t request_item = 0;
	static uint8_t spell_inventory = 0;

	int item_order = 0, spell_order = 0;
	int base_x, base_y;
	uint8_t which_cursor = 0;

	static int cursor_position[2];
	cursor_position[0] = 5; // x
	cursor_position[1] = 50;// y
	
	static int spell_cursor_position[2];
	spell_cursor_position[0] = 230;
	spell_cursor_position[1] = 50;

	struct item_check
	{
		int check;
	} spell_check[MAX_SPELLS], item_check[MAX_ITEM];


	SDL_Color font_red = { 0xff, 0x00, 0x00, 0x00 };
	SDL_Color font_fg = { 0xff, 0xff, 0xff, 0x00 };
	SDL_Color font_bg = { 0x00, 0x00, 0x00, 0x00 };
	SDL_Color green_fg = { 0x35, 0xcd, 0x06, 0x0};

	char *buf[MAX_ITEM];
	char *spell[MAX_SPELLS];
	
	SDL_Surface *use_item;
	SDL_Surface *sword_header, *armor_header, *spell_header;
	SDL_Surface *item_column, *spell_column; 

	extern SDL_Surface *cursor_image, *cursor_image_selected;
	SDL_SetAlpha (cursor_image, SDL_SRCALPHA, SDL_ALPHA_TRANSPARENT + 50);
	SDL_SetAlpha (cursor_image_selected, SDL_SRCALPHA, SDL_ALPHA_TRANSPARENT + 100);
	
	SDL_Surface *active_cursor[2];
	active_cursor[0] = cursor_image;
	active_cursor[1] = cursor_image_selected;
	
	
	char *action_item[3];
	
	for (i = 0; i < 3; i++)
		action_item[i] = (char *)malloc(255);
 
	for (i = 0; i < MAX_SPELLS; i++)
		spell[i] = (char *)malloc(255);
			
	/* ACTIVE ACTION ITEMS */
	sprintf(action_item[0], "%s - DMG: %d", action_item_list.sword[inventory->ActiveSword.id].name,
					      action_item_list.sword[inventory->ActiveSword.id].damage);

	sprintf(action_item[1], "%s - DMG: %d", action_item_list.spell[inventory->ActiveSpell.id].name,
					      action_item_list.spell[inventory->ActiveSpell.id].damage);

	sprintf(action_item[2], "%s - DMR: %d", action_item_list.armor[inventory->ActiveArmor.id].name,
					      action_item_list.armor[inventory->ActiveArmor.id].damage_intake);

	SDL_Surface *active_sword = TTF_RenderText_Blended (item_font, action_item[0], font_fg);
	SDL_Surface *active_armor = TTF_RenderText_Blended (item_font, action_item[2], font_fg);
	SDL_Surface *active_spell = TTF_RenderText_Blended (item_font, action_item[1], font_fg);

	/* The cursor needs to know what item its on */
	/* by the actual ID, otherwise we can't use items */

	for (i = 0; i < MAX_ITEM; i++)
	{
		item_cursor[i].item_id = 0;
		item_cursor[i].position[0] = 0;
		item_cursor[i].position[1] = 0;
	}
	
	for (i = 0; i < MAX_SPELLS; i++)
	{
		spell_item_cursor[i].spell_id = 0;
		spell_item_cursor[i].position[0] = 0;
		spell_item_cursor[i].position[1] = 0;
	}

	/* allocate and initialize! */
	for (i = 0; i < 32; i++)
	{
		buf[i] = (char *) malloc (255);
		item_check[i].check = 0;
	}

	for (i = 0; i < MAX_SPELLS; i++)
	{
		spell[i] = (char *) malloc(255);
		spell_check[i].check = 0;
	}
	if (cursor == 0) // || ITEM_COUNT == 0)
		goto skip_cursor;

	switch(cursor)
	{
		case CURSOR_UP:
			if (cursor_position[1] <= 50)
				/* Cannot move up anymore */
				break;
			cursor_position[1] -= 18;
			break;
		case CURSOR_RIGHT:
			/* The right column is exclusive to magic spells */
			spell_inventory = 1;
			break;
		case CURSOR_DOWN:
			if (cursor_position[1] >= 400)
				break;
			cursor_position[1] += 18;
			break;
		case CURSOR_LEFT:
			spell_inventory = 0;
			break;
	}

	skip_cursor:
	sprintf (buf[0], "%s", player);
	count++;

	for (i = 0; i < MAX_ITEM; i++)
	{
		switch (inventory->items[i].item_id)
		{
		case POTION_OF_LIFE_ID:
			item_cursor[item_order++].item_id = POTION_OF_LIFE_ID;
					
			if (item_check[POTION_OF_LIFE_ID].check)
				break;

			sprintf (buf[count++], "Potion of HP [10HP] Quanity: %d",
				 inventory->items[i].quanity);
			
			item_check[POTION_OF_LIFE_ID].check = 1;
			break;
		case POTION_OF_MAGIC_ID:
			item_cursor[item_order++].item_id = POTION_OF_MAGIC_ID;

			if (item_check[POTION_OF_MAGIC_ID].check)
				break;

			sprintf (buf[count++], "Potion of MP [10MP] Quanity: %d",
				 inventory->items[i].quanity);
		
			item_check[POTION_OF_MAGIC_ID].check = 1;
			break;
		}
	}

	for (i = 0; i < MAX_SPELLS; i++)
        {
                switch (action_item_list.spell[i].id)
                {
                case FLAME_SPELL:
			
			/* Do we have this spell yet? */
			if (action_item_list.spell[i].attained == 0)
				break;
	
                        spell_item_cursor[spell_order++].spell_id = FLAME_SPELL; 

                        if (spell_check[FLAME_SPELL].check)
                                break;

                        sprintf (spell[spell_count++], "Flame Spell DMG [%d]",
                                 action_item_list.spell[i].damage);

                        spell_check[FLAME_SPELL].check = 1;
                        break;
                case ICE_SPELL:

			if (action_item_list.spell[i].attained == 0)
                                break;

			spell_item_cursor[spell_order++].spell_id = ICE_SPELL;

                        if (spell_check[ICE_SPELL].check)
                                break;

                        sprintf (spell[spell_count++], "Ice Spell DMG [%d]",
                                 action_item_list.spell[i].damage);

                        spell_check[ICE_SPELL].check = 1;
                        break;
                }
        }

	/* !!! Finish binding cursor to items !!! */
	base_x = 5;
	base_y = 50;
	for (j = 1, i = 0; i < MAX_ITEM; i++, j++)
	{
		item_cursor[i].position[X] = base_x;
		item_cursor[i].position[Y] = base_y * j;
	}
	
	for (j = 1, i = 0; i < MAX_SPELLS; i++, j++)
	{
		spell_item_cursor[i].position[X] = 230;
		spell_item_cursor[i].position[Y] = base_y * j;
	}

	use_item = TTF_RenderText_Blended (inventory_font, "(U)se Item - (D)rop Item - (B)ack", font_fg);
	
	sword_header = TTF_RenderText_Blended (inventory_font, "Active Sword", green_fg);
	armor_header = TTF_RenderText_Blended (inventory_font, "Active Armor", green_fg);
	spell_header = TTF_RenderText_Blended (inventory_font, "Active Spell", green_fg);

	drawSprite(sword_header, screen, 0, 0, 460, 240, 20, 15);
	drawSprite(active_sword, screen, 0, 0, 460, 260, 20, 20);
	
	drawSprite(armor_header, screen, 0, 0, 460, 300, 20, 15);
	drawSprite(active_armor, screen, 0, 0, 460, 320, 20, 20);

	drawSprite(spell_header, screen, 0, 0, 460, 360, 20, 15);
	drawSprite(active_spell, screen, 0, 0, 460, 380, 20, 20);
	
	/* Header Message */
	ts[0].s = TTF_RenderText_Blended (inventory_font, buf[0], font_fg);
	drawSprite (ts[0].s, screen, 0, 0, 480, 70, 250, 10);
	
	DisplayStats(stats_font, *stats, screen);

	item_column = TTF_RenderText_Blended (inventory_font, "Item Bag", green_fg);
	spell_column = TTF_RenderText_Blended (inventory_font, "Spell Book", green_fg);
	
	drawSprite (item_column, screen, 0, 0, 5, 10, 50, 20);
	drawSprite (spell_column, screen, 0, 0, 230, 10, 50, 20);

	/* Display Left column (items) */
	for (i = 1; i < count; i++)
	{
		ts[i].s = TTF_RenderText_Blended (item_font, buf[i], font_fg);
		drawSprite (ts[i].s, screen, 0, 0, 5, 30 + i * 18, 250, 10);
	}

	for (j = 1, i = 0; i < spell_count; i++, j++)
	{
		ts2[i].s = TTF_RenderText_Blended (item_font, spell[i], font_fg);
		drawSprite (ts2[i].s, screen, 0, 0, 230, 30 + j * 18, 250, 10);
	}

       // drawSprite(cursor_image, screen, 0, 0, cursor_position[0], cursor_position[1], 250, 20);

	
        switch(action)
        {
                case USE_ITEM:
                        for (i = 0; i < MAX_ITEM; i++)
                                if (cursor_position[X] == item_cursor[i].position[X] &&
                                    cursor_position[Y] == item_cursor[i].position[Y])
                                {
                                        for (j = 0; j < MAX_ITEM; j++)
                                                if (inventory->items[j].item_id == item_cursor[i].item_id)
                                                {
							switch(item_cursor[i].item_id)
							{
								case POTION_OF_LIFE_ID:
									stats->hp += 10;
									if (stats->hp > stats->max_hp)
										stats->hp = 10; //stats->max_hp;
									break;
								case POTION_OF_MAGIC_ID:
									stats->mp += 10;
									if (stats->mp > stats->max_mp)
										stats->mp = stats->max_mp;
									break;
							}
                                                        if (inventory->items[j].quanity == 1)
                                                        {
                                                                inventory->items[j].item_id = 0;
                                                                inventory->items[j].quanity = 0;
                                                        }
                                                        else
                                                                inventory->items[j].quanity--;
							action = 0;
							request_item = 0;
							ITEM_COUNT--;
                                                        break;
                                                } 
					break;
                                }
			INVENTORY_UPDATE = 1;
                        break;
                case DROP_ITEM:
                for (i = 0; i < MAX_ITEM; i++)
                                if (cursor_position[X] == item_cursor[i].position[X] &&
                                    cursor_position[Y] == item_cursor[i].position[Y])
                                {
                                        for (j = 0; j < MAX_ITEM; j++)
                                                if (inventory->items[j].item_id == item_cursor[i].item_id)
                                                {
							
						        switch(item_cursor[i].item_id)
                                                        {
                                                                case POTION_OF_LIFE_ID:
                                                                        stats->hp += 10;
								        if (stats->hp > stats->max_hp)
                                                                  	      stats->hp = stats->max_hp;
                                                                        break;
                                                                case POTION_OF_MAGIC_ID:
                                                                        stats->mp += 10;
                                                                        if (stats->mp > stats->max_mp)
                                                                                stats->mp = stats->max_mp;
                                                                        break;
                                                        }

                                                        if (inventory->items[j].quanity == 1)
                                                        {
                                                                inventory->items[j].item_id = 0;
                                                                inventory->items[j].quanity = 0;
                                                        }
                                                        else
                                                                inventory->items[j].quanity--;
							action = 0;
							request_item = 0;
							ITEM_COUNT--;
                                                        break;
                                                }
					break;
                                }
			 INVENTORY_UPDATE = 1;
               		 break;
                case GO_BACK:
                        request_item = 0;
                        break;
		case REQUEST_ITEM:
			request_item = 1;
			break;

        }
	
	if (request_item)
	{
		which_cursor = 1;
	 	drawSprite(use_item, screen, 0, 0, 200, 440, 250, 20);

	}

	if (!spell_inventory)
		drawSprite(active_cursor[which_cursor], screen, 0, 0, cursor_position[0], cursor_position[1], 250, 20);
	else
		drawSprite(active_cursor[which_cursor], screen, 0, 0, spell_cursor_position[0], spell_cursor_position[1], 250, 20);
	drawSprite(lavren, screen, 0, 0, 460, 100, 173, 133);
	
	SDL_Flip (screen);


	for (i = 0; i < count; i++)
		SDL_FreeSurface(ts[i].s);

	SDL_FreeSurface(sword_header);
	SDL_FreeSurface(armor_header);
	SDL_FreeSurface(spell_header);

	SDL_FreeSurface(active_sword);
	SDL_FreeSurface(active_armor);
	SDL_FreeSurface(active_spell);

	for (i = 0; i < 3; i++)
		free (action_item[i]);

	for (i = 0; i < 8; i++)
		free (buf[i]);

}
