
#include <SDL/SDL.h>

#include "SDL_collide.h"
#include "bmp_sizes.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_image.h"
#include "sounds.h"
#include "socket.h"
#include <SDL/SDL_gfxPrimitives.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <zlib.h>
#include <png.h>
#include <pngconf.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/mman.h>

#define COLOR_DEPTH 16
/* Master game script */
#define MASTER_DATA "data/master.gs"

#define NORTH 0
#define NE 1
#define EAST 2
#define SE 3
#define SOUTH 4
#define SW 5
#define WEST 6
#define NW 7

#define UP 0
#define RIGHT 2
#define DOWN 4
#define LEFT 6

#define CURSOR_UP  1
#define CURSOR_RIGHT 2 
#define CURSOR_DOWN 3
#define CURSOR_LEFT 4

#define SAVE_MAGIC "x53-x41-x56-x45-x5f-x4d-x41-x47-x49-x43\n"

#define SOLID 0
#define SHADED 1
#define BLENDED 2

#define ALREADY_LOADED 1

#define SCREEN_W 800
#define SCREEN_H 600

#define FRAMES_PER_SECOND 100

/* ITEM ID's */
#define POTION_OF_LIFE_ID 1
#define POTION_OF_MAGIC_ID 2

#define VOLUME_PER_SOUND SDL_MIX_MAXVOLUME >> 1

/* FONT */
#define FONT "fonts/FreeSans.ttf"
#define FONT_SIZE 12

#define INTRO_FONT "fonts/FreeSans.ttf"

#define FONT2 "fonts/FreeSans.ttf"
#define FONT2_SIZE 16

#define STATS_FONT "fonts/FreeSerif.ttf"

#define SAVE_PATH "save_data/"

#define SWORD_TIMING 160000 //usleep of how long sword stays out

#define MAX_SPELLS 6

#define FLAME_SPELL 0
#define ICE_SPELL 1
#define IMMATERIALIZE 2

/* Map frames */
#define MAP_FRAME1 	20
#define MAP_FRAME1_A 	21
#define MAP_FRAME1_B 	22
#define MAP_FRAME1_C	23
#define MAP_FRAME1_D	24
#define MAP_FRAME1_E	25
#define MAP_FRAME1_F	26
#define MAP_FRAME1_G	27
#define MAP_FRAME1_H	28
#define MAP_FRAME1_I	29
#define MAP_FRAME1_J	30
#define MAP_FRAME1_K    31
#define MAP_FRAME1_L	32
#define MAP_FRAME1_M	33
#define MAP_FRAME1_N	34
#define MAP_FRAME1_O	35
#define MAP_FRAME1_P	36
#define MAP_FRAME1_T	40
#define MAP_FRAME1_Z	46
/* House Frames EVEN #*/
#define HOUSE_FLOOR1 10
#define HOUSE_FLOOR2 11

#define ONCE_EVERY_LOOP 1 /* For Slow CPU */
#define ONCE_EVERY_2_LOOPS 2 /* For Fast CPU */

/* The bitmap of the man vs. the man with his sword */
/* is offset by about 30, if we don't use this it makes the */
/* player look like he's jumping back when he uses his sword */
#define SWORD_BMP_OFFSET 25


/* The starting sword does damage of 1 HP + LVL */
#define START_SWORD_DAMAGE 1

#define START_SPELL_DAMAGE 2
/* HEALTH */
#define START_HP 10

/* MAGIC */
#define START_MP 5

/* These values are computed with the player level */
/* to get the final value */ 
#define FLAME_DAMAGE 2
#define ICE_DAMAGE 3

#define FLAME_SPELL 0
#define ICE_SPELL 1

#define FLAME_W 13
#define FLAME_H 27

#define EVIL_PLANT_1 0
#define CYCLOPS 1

/* Separate images for its color changes */
/* when it gets attacked */
#define EVIL_PLANT_1_A 0
#define EVIL_PLANT_1_B 1
#define EVIL_PLANT_1_C 2
#define EVIL_PLANT_1_D 3
#define EVIL_PLANT_1_E 4 
#define EVIL_PLANT_1_F 5 
#define EVIL_PLANT_1_G 6 
#define EVIL_PLANT_1_H 7
#define EVIL_PLANT_1_I 8

/* Starting coordinates for player on map */
#define FROM_RIGHT 1
#define FROM_LEFT  2
#define FROM_BOTTOM 3
#define FROM_TOP 4

#define MAX_ITEM 32
/* bitmap surfaces of magic and health */

#define MP_TICK_MAX 1000

#define X 0
#define Y 1

#define USE_ITEM 1
#define DROP_ITEM 2 
#define GO_BACK 3
#define REQUEST_ITEM 4

#define PENDING_QUEST 0
#define ADDED_QUEST 1

#define JMP_SHIFT 1 /* Determines the number of bitwise left shifts 
		       for when a character jumps back in coordinates */

/* Modulo values primarily used for enemy speed */
/* i.e if val % modulo_value == 0 then move animation frame */
#define FAST_SPEED_MODULO 5
#define MEDIUM_SPEED_MODULO 15
#define SLOW_SPEED_MODULO 25

/* Bounding Box Collision Detection OR Perfect Pixel Collision detection */
/* These can be passed to SDL_CollideBox() function/wrapper */
#define BBOX 1
#define PIXEL 2

#define MAX_LINE 512

#define OBJECT 1
#define ENEMY 2

/* for loaded sounds */
typedef struct sound_s {
uint8_t *samples;
uint32_t length;
} sound_t, *sound_p;

/* for actively playing sounds */
typedef struct playing_s {
int active;
sound_p sound;
uint32_t position;
} playing_t, *playing_p;


struct sword_item
{
	int id;
	int damage;
	char *name;
};

struct armor_item
{
	int id;
	int damage_intake;
	char *name;
};

struct spell_item
{
#define ELEMENTAL_DAMAGE 0
#define MYSTICAL 2
	int type;
	int id;
	int damage;
	int rmp;
	char *name;
	int attained;
}; 

struct action_item_list
{	
	struct sword_item sword[12];
	struct armor_item armor[12];
	struct spell_item spell[12];
};
	
enum textquality {solid, shaded, blended};

struct magic_image
{
	SDL_Surface *flame;
	SDL_Surface *ice;
};

/* MP AND HP */
struct player_stats
{
	uint8_t level;
	uint32_t NextLevel;
	uint32_t mp;
	uint32_t max_mp;
	uint32_t hp;
	uint32_t max_hp;
	uint32_t gold;
	uint32_t exp;

};

struct active_spell
{
	int id; /* spell number */
	int rmp; /*required mp */
	int type;
	int damage; /* inflicted damage */
};


struct active_sword
{
	int id; /* sword number */
	int damage; /* inflicted damage */
};
	
struct active_armor
{
	int type;
	int id;
	int damage_intake; /* how it computes hit point intake */
};

struct active_items
{
	uint32_t item_id;
	uint32_t quanity;
};

struct quest_list
{
	char name[255];
	int id;
	int status;
	struct quest_list *next;
};

struct inventory
{
	struct active_items items[64];
	struct active_spell ActiveSpell;
	struct active_sword ActiveSword;
	struct active_armor ActiveArmor;
	struct quest_list *root;
	
};

struct save_data
{
	int x;
	int y;

	struct player_stats stats;
	struct active_items item[MAX_ITEM];
	int item_count;
	int next_level;
	struct active_sword ActiveSword;
	struct active_spell ActiveSpell;
	struct active_armor ActiveArmor;
	int ActiveWorld;
};

struct monster_size
{
	int w;
	int h;
};

struct NPC_size
{
	int w;
	int h;
};

struct hero_size
{
	int w, h;
};

struct dialog_info
{
	int index;
	char *speech[32];
};

struct NPC
{
	int InAction; /* NPC is talking */
	int id;
        char name[255];
	char image_key[255];
	int quests[32];
	int quest_count;
        int item;
        int location;
        int x, y;
	int status;
	int image_angles;
	int talk_images;
	int direction;
	char image_directory[256];
        SDL_Surface *image[8][32];
	struct NPC_size size[4];
        int amv, amc;
	int count;
};


/* The monster/enemy struct is huge, but faciliates */
/* the necessary values for monsters to have atleast */
/* some level of complexity */
struct MonstXY
{
	/* north, south, east, west */
	#define N 0
	#define E 1
	#define S 2
	#define W 3
	
	int x;
	int y;
	int prevX;
	int prevY;
	int timer_count;
	int hit; // is he hit (for non animation characters */
	int type;
	int status; /* alive, dead? */
	SDL_Surface *image, *orig_image;
	SDL_Surface *images[6]; /* For non-multi-directional animation */

	SDL_Surface *animation[4][9]; /* multi-dir walk animation */
	SDL_Surface *animation_hit[4][9]; /* show hit animation */
	SDL_Surface *animation_attack[4][9]; /* show attack animation */
	uint8_t hit_count; /* count through hit frames */
	int hit_amc; /* hit modulo count */
	int hit_amv; /* hit modulo value */

	int attack_amc; /* attack modulo count */
	int attack_amv; /* attack modulo value */

	uint8_t attacked; /* currently attacked? */
	struct monster_size size[4];

	uint8_t direction; /* Current direction */
	uint8_t multi_directional; /* YES/NO */
	int image_count; /* Number of images for walking animation */
	int image_index; /* current index for them */
	int hit_image_count; /* number of images for hit animation */ 
	int hit_image_index; /* obvious */ 
	int attack_image_count; /* # of images for attack animation */
	int attack_image_index; /* obvious */
	int amc; /* animation modulo count */
	int amv; /* animation modulo value */
	
	SDL_Surface *hit_image[10]; /* Hit image for non-multi-dir animation */
	/* WIDTH/HEIGHT */
	int w;
	int h;
	/* move_count is a counter used with CPU_SPEED_VALUE */
	/* To calculate the monsters movements */
	uint32_t move_count;
	int hp;
	int max_hp;
	int damage;
	/* Last move direction */
	int LastMove;
};

/* We need precision in image size for hero */
struct surface_angle
{
	SDL_Surface *image[8];
	struct hero_size sz[8];/* w/h of each image */
};

struct hero
{
	int x, y;
	int attack_amc, attack_amv;
	int spell_amc, spell_amv;
	int amc, amv;
	
	struct surface_angle walk[16];
	struct surface_angle walk_no_sword[8];
	struct surface_angle attack[8];
	struct surface_angle spell[8];
	
	uint8_t attacking;
	int walk_count;
	int attack_count;
	
	int walk_index;
	int attack_index;

	int w;
	int h; 

};
struct text_surface
{
	SDL_Surface *s;		
};

struct merchant_info
{
	int MerchantID;
	int pol_count;
	int pom_count;
};

struct Particles
{
	int x, y;
};


typedef struct {
unsigned short type;
unsigned long size;
unsigned short reserved1;
unsigned short reserved2;
unsigned long offsetbits;
} bmp_hdr;

typedef struct {
   unsigned long size;
   unsigned long width;
   unsigned long height;
   unsigned short planes;
   unsigned short bitcount;
   unsigned long compression;
   unsigned long sizeimage;
   long xpelspermeter;
   long ypelspermeter;
   unsigned long colorsused;
   unsigned long colorsimportant;
} bmp_info;

typedef struct {
   unsigned char blue;
   unsigned char green;
   unsigned char red;
} bmp_rgb;

int GetAttackSpeed(void);
void DrawHero(struct hero *);
int SPIRIT_X(int);
int SPIRIT_Y(int);
int LoadHero(struct hero *);
void talk2NPC(int, int, struct dialog_info *dialogs);
void SetEnemyCoords(struct MonstXY *, int, int);
void collect_save_data(struct save_data *, int, int);
void init_NPC(int, struct dialog_info *);
int add_quest(char *, int);
int get_bmp_size(char *, int *, int *);
void do_panic(void);
int parse_dynamic_data(struct MonstXY *, struct NPC *, struct inventory *, struct dialog_info *, int);
int PlaySound(sound_p);
int LoadAndConvertSound(char *, SDL_AudioSpec *, sound_p);
void AudioCallback(void *, uint8_t *, int);
void ClearPlayingSounds(void);
int S2SO_X(int);
int S2SO_Y(int);
int S2BO_X(int);
int S2BO_Y(int);
int T2RO_X(int);
int T2RO_Y(int);
int L2SO_X(int);
int L2SO_Y(int);
int L2BO_X(int);
int L2BO_Y(int);
void init_action_items(struct action_item_list *);
void ClearBlack(SDL_Surface *);
void talk2merchant(int, struct player_stats *);
bool DropCoin(int, int);
bool testKey(SDLKey key);
void DrawParticles(int);
void init_fog(void);
void init_particles(int, int, int);
void SetOpaque(SDL_Surface *);
uint32_t GetTicks(uint32_t);
void LoadImages(void);
int HitEnemy(SDL_Surface **, int, int, SDL_Surface *, int, int);
int InBounds(int, int, int);
int EnemyInBounds(SDL_Surface *, int, int, int, int);
int HitHouse(int, int, int);
void init_monsters(int);
void DrawWorld(int);
void DrawHouse(void);
/* Here is a wrapper that allows to pass an array of surfaces, elements specified by arg 7 */
int SDL_CollideBox(SDL_Surface **, int, int,
                   SDL_Surface *, int, int, int, int);
void JumpBack(int);
void draw_NPC(int, int, int, int);
uint32_t SetExp(int, int, struct player_stats *);
void GameOver(TTF_Font *, SDL_Surface *);
SDL_Surface* LoadImage(const char *);
SDL_Surface* LoadAlphaImage(const char *);
SDL_Surface * xSDL_LoadBMP (const char *);
SDL_Surface * xSDL_SetVideoMode (int, int, int, uint32_t);
void xSDL_Init (uint32_t);
void xSDL_Quit (void);
void drawSprite(SDL_Surface *,
                SDL_Surface *,
                int, int, int, int, int, int);
void ResetCords(int *x, int *y, int StartPoint);
void ClearCords(int *, int *, int *, int *);
int GetSpeedIncr(void);
void cleanup(void);
int save_game(int, struct save_data);
int load_game(char *, struct save_data *);
SDL_Surface *drawtext(TTF_Font *, SDL_Color, SDL_Color, char *, int);
int ShowInventory(TTF_Font *, struct merchant_info, SDL_Surface *);
void DisplayStats(TTF_Font *font, struct player_stats stats, SDL_Surface *);
int WhichFrame(int ActiveFrame, int *x, int *y);
int ListInventory(TTF_Font *, TTF_Font *, struct inventory *, char *, SDL_Surface *screen, int8_t, int8_t, struct player_stats *);
