
#include "rpg.h"

/* The code in this src file was borrowed from the book "Programming Linux Games" */

extern playing_t playing[2];	
extern int MAX_SOUND_EFFECTS;

void AudioCallback(void *user_data, uint8_t *audio, int length)
{
	int i;

	memset(audio, 0, length);
	
	for (i = 0; i < 2; i++)
		if (playing[i].active)
		{
			uint8_t *sound_buf;
			uint32_t sound_len;
			
			sound_buf = playing[i].sound->samples;
			sound_buf += playing[i].position;
			
			if ((playing[i].position + length) > playing[i].sound->length)
				sound_len = playing[i].sound->length - playing[i].position;
			else
				sound_len = length;
			
			SDL_MixAudio(audio, sound_buf, sound_len, VOLUME_PER_SOUND);
			playing[i].position += length;
			
			if (playing[i].position >= playing[i].sound->length)
				playing[i].active = 0;

		}
}

int LoadAndConvertSound(char *filename, SDL_AudioSpec *spec, sound_p sound)
{
	SDL_AudioCVT cvt;
	SDL_AudioSpec loaded;
	uint8_t *new_buf;

	if (SDL_LoadWAV(filename, &loaded, &sound->samples, &sound->length) == NULL)
	{
		printf("Unable to load sound: %s\n", SDL_GetError());
		return 1;
	}
	
	if (SDL_BuildAudioCVT(&cvt, loaded.format, loaded.channels, loaded.freq,
			      spec->format, spec->channels, spec->freq) < 0)
	{
		printf("Unable to convert sound: %s\n", SDL_GetError());
		return 1;
	}

	cvt.len = sound->length;
	new_buf = (uint8_t *)malloc(cvt.len * cvt.len_mult);
	if (new_buf == NULL)
	{
		printf("Memory allocation failed\n");
		SDL_FreeWAV(sound->samples);
		return 1;
	}

	memcpy(new_buf, sound->samples, sound->length);

	cvt.buf = new_buf;
	if (SDL_ConvertAudio(&cvt) < 0)
	{
		printf("Audio conversion error: %s\n", SDL_GetError());
		free(new_buf);
		SDL_FreeWAV(sound->samples);
		return 1;
	}

	SDL_FreeWAV(sound->samples);
	sound->samples = new_buf;
	sound->length = sound->length * cvt.len_mult;
	
	return 0;
}

void ClearPlayingSounds(void)
{
	int i;
	
	for (i = 0; i < MAX_SOUND_EFFECTS; i++)
		playing[i].active = 0;

}

int PlaySound(sound_p sound)
{
	int i;

	for(i = 0; i < MAX_SOUND_EFFECTS; i++)				
		if (playing[i].active == 0)
			break;
	
	if (i == MAX_SOUND_EFFECTS)
		return 1;
		
	SDL_LockAudio();
	playing[i].active = 1;
	playing[i].sound = sound;
	playing[i].position = 0;
	SDL_UnlockAudio();
	return 0;
}



