/*
 * sprite.c
 * This source code is a part of RPG    
 * <ryan@bitlackeys.com>
 */

#include "rpg.h"

SDL_Surface *
LoadImage (const char *filename)
{
	SDL_Surface *loaded = xSDL_LoadBMP (filename);
	SDL_Surface *optimal = NULL;
	if (loaded)
	{
		SDL_SetColorKey (loaded, SDL_RLEACCEL | SDL_SRCCOLORKEY, SDL_MapRGB (loaded->format, 0xFF, 0, 0xFF));	/* magenta is our transprent */
		optimal = SDL_DisplayFormat (loaded);
		SDL_FreeSurface (loaded);
	}
	if (!optimal)
	{
		printf("Unable to load image: %s\n", filename);
		exit(-1);
	}
	return optimal;
}

SDL_Surface *
LoadAlphaImage (const char *filename)
{
	printf ("Attempting to open: %s\n", filename);

	SDL_Surface *loaded;
	SDL_Surface *optimal = NULL;

	loaded = IMG_Load (filename);
	if (loaded == NULL)
	{
		printf ("IMG_Load failed: %s\n", IMG_GetError ());
		goto failure;
	}
	if (loaded)
	{
		if (SDL_SetColorKey (loaded, SDL_RLEACCEL | SDL_SRCCOLORKEY,
												 SDL_MapRGB (loaded->format, 0xFF, 0, 0xFF)) == -1)
		{
			printf ("SetColorKey failed\n");
			goto failure;
		}
		if ((optimal = SDL_DisplayFormat (loaded)) == NULL)
		{
			printf ("SDL_DisplayFormatAlpha failed\n");
			goto failure;
		}
		SDL_FreeSurface (loaded);
	}

	if (!optimal)
	{
	failure:
		printf ("Error loading Alpha image\n");
		exit (-1);
	}
	return optimal;
}

void
drawSprite (SDL_Surface * imageSurface,
						SDL_Surface * screenSurface,
						int srcX, int srcY, int dstX, int dstY, int width, int height)
{

	SDL_Rect srcRect;
	srcRect.x = srcX;
	srcRect.y = srcY;
	srcRect.w = width;
	srcRect.h = height;

	SDL_Rect dstRect;
	dstRect.x = dstX;
	dstRect.y = dstY;
	dstRect.w = width;
	dstRect.h = height;


	//SDL_SetColorKey(imageSurface, SD_RLEACCEL | SDL_SRCCOLORKEY, SDL_MapRGB(imageSurface->format, 255, 0, 255));

	if (srcX == 0 && srcY == 0 && dstX != 0 && dstY != 0)
		SDL_BlitSurface (imageSurface, NULL, screenSurface, &dstRect);
	else if (srcX == 0 && srcY == 0 && dstX == 0 && dstY == 0)
		SDL_BlitSurface (imageSurface, NULL, screenSurface, NULL);
	else
		SDL_BlitSurface (imageSurface, &srcRect, screenSurface, &dstRect);

}
